@@ -12,7 +12,7 @@ def w56_download_by_id(id, title = None, output_dir = '.', merge = True, info_on
     assert title
     hd = info['hd']
     assert hd in (0, 1, 2)
-    hd_types = [['normal', 'qvga'], ['clear', 'vga'], ['super', 'dvga']][hd]
+    hd_types = [['normal', 'qvga'], ['clear', 'vga'], ['super', 'wvga']][hd]
     files = [x for x in info['rfiles'] if x['type'] in hd_types]
     assert len(files) == 1
     size = int(files[0]['filesize'])
